/// <reference types="node" />
import { IncomingHttpHeaders as IncomingHttpHeadersH1 } from "http";
import { IncomingHttpHeaders as IncomingHttpHeadersH2 } from "http2";
export declare type IncomingHttpHeaders = IncomingHttpHeadersH1 | IncomingHttpHeadersH2;
