import { FetchInit } from "./core";
import { SimpleSessionHttp2 } from "./simple-session";
import { FetchExtra } from "./fetch-common";
import { Request } from "./request";
import { Response } from "./response";
export declare function fetch(session: SimpleSessionHttp2, input: Request, init?: Partial<FetchInit>, extra?: FetchExtra): Promise<Response>;
